/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: gfx_debug.c,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file gfx_debug.c
 *  gfx driver debug interface
 *  
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "gfx_if.h"
#include "gfx_drvif.h"
//#include "gfx_dif.h"
#include "gfx_hw.h"
#include "gfx_cmdque.h"
#include "gfx_debug.h"

#if defined(GFX_DEBUG_MODE)
//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------

// gfx statistics for debug use
static GFX_DBG_INFO_T _rGfxDbgInfo;

static HAL_TIME_T _rGfxDbgFlushStartTime;

static GFX_DBG_ENUM2TEXT_T _rGfxDbgClkEnum2TextTbl[] =
{
    {   (INT32)E_GFX_CLK_XTAL_CK, "xtal_clk"},
    {   (INT32)E_GFX_CLK_SYSPLL_D2, "syspll / 2"},
    {   (INT32)E_GFX_CLK_TVDPLL_D3, "tvdpll / 3"},
    {   (INT32)E_GFX_CLK_TVDPLL_D5, "tvdpll / 5"},
    {   (INT32)E_GFX_CLK_DTDPLL_D3, "dtdpll / 3"},
    {   (INT32)E_GFX_CLK_DTDPLL_D4, "dtdpll / 4"},
    {   (INT32)E_GFX_CLK_PSPLL_D1, "pspll / 1"},
    {   (INT32)E_GFX_CLK_MEM, "mempll"},

    {   (INT32)E_GFX_CLK_MAX, NULL}
};

//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------


//-------------------------------------------------------------------------
/** GFX_DbgInit
 *  
 */
//-------------------------------------------------------------------------
void GFX_DbgInit(void)
{
    _rGfxDbgInfo.i4DrvFlushCount = 0;
    _rGfxDbgInfo.i4DrvIntCount = 0;
    _rGfxDbgInfo.i4MwFlushCount = 0;
    _rGfxDbgInfo.i4MwIntCount = 0;
    _rGfxDbgInfo.i4CmFlushCount = 0;
    _rGfxDbgInfo.i4CmIntCount = 0;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncDrvFlushCount
 *  increase driver flush count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncDrvFlushCount(void)
{
    _rGfxDbgInfo.i4DrvFlushCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncDrvIntCount
 *  increase driver interrupt count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncDrvIntCount(void)
{
    _rGfxDbgInfo.i4DrvIntCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncMwFlushCount
 *  increase middleware flush count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncMwFlushCount(void)
{
    _rGfxDbgInfo.i4MwFlushCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncMwIntCount
 *  increase middleware interrupt count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncMwIntCount(void)
{
    _rGfxDbgInfo.i4MwIntCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncCmFlushCount
 *  increase c-model flush count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncCmFlushCount(void)
{
    _rGfxDbgInfo.i4CmFlushCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncCmIntCount
 *  increase c-model interrupt count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncCmIntCount(void)
{
    _rGfxDbgInfo.i4CmIntCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgIncEngineCrashCount
 *  increase engine crash count
 */
//-------------------------------------------------------------------------
void GFX_DbgIncEngineCrashCount(void)
{
    _rGfxDbgInfo.i4EngineCrashCount++;
}

//-------------------------------------------------------------------------
/** GFX_DbgDumpInfo
 *  dump gfx debug information
 */
//-------------------------------------------------------------------------
void GFX_DbgDumpInfo(void)
{
    CHAR szClkSrcText[GFX_STR_SIZE];

    LOG(1, "\n--- gfx dbg report ---- (begin)\n");

    LOG(1, "   driver     flush     count = %d\n", _rGfxDbgInfo.i4DrvFlushCount);
    LOG(1, "   driver     interrupt count = %d\n", _rGfxDbgInfo.i4DrvIntCount);
    LOG(1, "   middleware flush     count = %d\n", _rGfxDbgInfo.i4MwFlushCount);
    LOG(1, "   middleware interrupt count = %d\n", _rGfxDbgInfo.i4MwIntCount);
    LOG(1, "   c-model    flush     count = %d\n", _rGfxDbgInfo.i4CmFlushCount);
    LOG(1, "   c-model    interrupt count = %d\n", _rGfxDbgInfo.i4CmIntCount);
    LOG(1, "   engine     crash     count = %d\n", _rGfxDbgInfo.i4EngineCrashCount);

    switch (_rGfxDbgInfo.i4DrvCurrentStatus)
    {
        case ((INT32)E_GFX_IDLE):
        LOG(1, "   driver     current  status = IDLE\n"); break;
        case ((INT32)E_GFX_BUSY):
        LOG(1, "   driver     current  status = BUSY\n"); break;
        case ((INT32)E_GFX_SEMIIDLE):
        LOG(1, "   driver     current  status = SEMIIDLE\n"); break;
        case ((INT32)E_GFX_CRASHED):
        LOG(1, "   driver     current  status = CRASHED\n"); break;
        case ((INT32)E_GFX_UNKNOWN):
        LOG(1, "   driver     current  status = UNKNOWN\n"); break;
        default:
        break;
    } // ~switch


    if (_rGfxDbgInfo.i4EngineExecuteMod == (INT32)E_GFX_SW_MOD)
    {
        LOG(1, "   engine     execute     mod = Software (C-model)\n");
    }
    else
    {
        LOG(1, "   engine     execute     mod = Hardware\n");
    }

    // get engine clock source text
    GFX_DbgEnum2Text(_rGfxDbgInfo.i4EngineClockSrc, szClkSrcText,
            _rGfxDbgClkEnum2TextTbl, (INT32)E_GFX_CLK_MAX);
    LOG(1, "   engine     clock    source = (%s)\n", szClkSrcText);

    LOG(1, "   gfx engine fire count = %u\n",
            (GFX_READ32(GFX_REG_0x40E0) & 0x0000ffff));
    LOG(1, "   gfx engine cmdq index = %u\n",
            ((GFX_READ32(GFX_REG_0x40E4) & 0x3fff0000) >> 16));
    LOG(1, "   gfx engine read  count of dram word = %u\n", GFX_READ32(GFX_REG_0x40EC));
    LOG(1, "   gfx engine write count of dram word = %u\n", GFX_READ32(GFX_REG_0x40F0));

    LOG(1, "--- gfx dbg report ---- (end)\n\n");
}

//-------------------------------------------------------------------------
/** GFX_DbgRecCurrentStatus
 *  record driver code current status
 */
//-------------------------------------------------------------------------
void GFX_DbgRecCurrentStatus(INT32 i4Status)
{
    _rGfxDbgInfo.i4DrvCurrentStatus = i4Status;
}

//-------------------------------------------------------------------------
/** GFX_DbgRecoverEngine
 *  Recover gfx engine when gfx engine crashed
 */
//-------------------------------------------------------------------------
void GFX_DbgRecoverEngine(void)
{
    GFX_DbgIncEngineCrashCount();
    GFX_DbgRecCurrentStatus((INT32)E_GFX_CRASHED);

    GFX_DbgShowBusyTime();

#if !defined(GFX_RISC_MODE) // cmdque mode
    GFX_HwISR();
#endif

    GFX_Init();

    // unlock cmdque resource
    GFX_UnlockCmdque();
}

//-------------------------------------------------------------------------
/** GFX_DbgRecFlushTime
 *  record flush start time
 */
//-------------------------------------------------------------------------
void GFX_DbgRecFlushTime(void)
{
    HAL_GetTime(&_rGfxDbgFlushStartTime);
}

//-------------------------------------------------------------------------
/** GFX_DbgShowBusyTime
 *  show busy working time
 */
//-------------------------------------------------------------------------
void GFX_DbgShowBusyTime(void)
{
    HAL_TIME_T rCurrentTime;
    HAL_TIME_T rDiffTime;

    HAL_GetTime(&rCurrentTime);
    HAL_GetDeltaTime(&rDiffTime, &_rGfxDbgFlushStartTime, &rCurrentTime);

    LOG(1, "GFX engine busy working time = %u sec\n", rDiffTime.u4Seconds);
    LOG(1, "-- flush   time = %u sec\n", _rGfxDbgFlushStartTime.u4Seconds);
    LOG(1, "-- current time = %u sec\n", rCurrentTime.u4Seconds);
}

//-------------------------------------------------------------------------
/** GFX_DbgRecEngExecuteMod
 *  record engine mode, HW-engine or SW-cmodel
 */
//-------------------------------------------------------------------------
void GFX_DbgRecEngExecuteMod(INT32 i4Mod)
{
    _rGfxDbgInfo.i4EngineExecuteMod = i4Mod;
}

//-------------------------------------------------------------------------
/** GFX_DbgEnum2Text
 *  translate enum into text
 */
//-------------------------------------------------------------------------
/*lint -save -e613 -e818 -e830 -e831  */
void GFX_DbgEnum2Text(INT32 i4Enum, CHAR *szText,
        GFX_DBG_ENUM2TEXT_T *prEnum2Text, INT32 i4Stop)
{
    GFX_ASSERT(prEnum2Text != NULL);
    GFX_ASSERT(szText != NULL);

    while (1)
    {
        if (prEnum2Text->i4Value == i4Stop)
        {
            LOG(0, "Error: Enum can NOT be translated into Text\n");
            GFX_ASSERT(prEnum2Text->i4Value != i4Stop);
        }

        if (i4Enum == prEnum2Text->i4Value)
        {
            break;
        }
        else
        {
            prEnum2Text++;
        }
    }

    GFX_UNUSED_RET(x_strcpy(szText, prEnum2Text->szText));
}
/*lint -restore */

//-------------------------------------------------------------------------
/** GFX_DbgRecEngClkSrc
 *  record engine clock source
 */
//-------------------------------------------------------------------------
void GFX_DbgRecEngClkSrc(INT32 i4ClkSrc)
{
    _rGfxDbgInfo.i4EngineClockSrc = i4ClkSrc;
}

//-------------------------------------------------------------------------
/** GFX_DbgCheckEngIdle
 *  check engine idle
 */
//-------------------------------------------------------------------------
void GFX_DbgCheckEngIdle(void)
{
    GFX_ASSERT(GFX_HwGetIdle());
    GFX_ASSERT(_rGfxDbgInfo.i4DrvFlushCount == _rGfxDbgInfo.i4DrvIntCount);
}
#endif // #if defined(GFX_DEBUG_MODE)
