/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/

#ifndef X_HAL_926_H
#define X_HAL_926_H

#include "x_typedef.h"
#include "x_lint.h"

//===========================================================================
// IRQ/FIQ related functions

externC void HalEnableIRQ(void);
externC void HalDisableIRQ(void);
externC void HalEnableFIQ(void);
externC void HalDisableFIQ(void);
externC UINT32 HalCriticalStart(void);
externC UINT32 HalCriticalSemiStart(void);
externC void HalCriticalEnd(UINT32 u4Flags);

//===========================================================================
// Cache related functions

externC BOOL HalIsICacheEnabled(void);
externC BOOL HalIsDCacheEnabled(void);
externC UINT32 HalGetICacheSize(void);
externC UINT32 HalGetDCacheSize(void);
externC UINT32 HalGetICacheLineSize(void);
externC UINT32 HalGetDCacheLineSize(void);
externC void HalEnableICache(void);
externC void HalDisableICache(void);
externC void HalEnableDCache(void);
externC void HalDisableDCache(void);
externC void HalEnableCaches(void);
externC void HalDisableCaches(void);
externC void HalInvalidateAllCaches(void);
externC void HalInvalidateICache(void);
externC void HalInvalidateICacheSingleLine(UINT32 u4Addr);
externC void HalInvalidateDCache(void);
externC void HalInvalidateDCacheSingleLine(UINT32 u4Addr);
externC void HalFlushDCacheSingleLine(UINT32 u4Addr);
externC void HalFlushInvalidateDCacheSingleLine(UINT32 u4Addr);
externC void HalFlushDCache(void);
externC void HalFlushInvalidateDCache(void);
externC void HalFlushWriteBuffer(void);
externC void HalInstructionMemoryBarrier(void);

//===========================================================================
// MMU related functions

externC BOOL HalIsMMUEnabled(void);
externC void HalEnableMMU(void);
externC void HalDisableMMU(void);
externC void HalInvalidateTLB(void);
externC BOOL HalInitMMU(UINT32 u4Addr);

//===========================================================================
// TCM mode

externC void HalSetInstTCMReg(UINT32 u4BaseAddr, UINT32 u4Size, BOOL fgEnable);
externC void HalSetDataTCMReg(UINT32 u4BaseAddr, UINT32 u4Size, BOOL fgEnable);


//===========================================================================
// Sleep mode

externC void HalWaitForInterrupt(void);

//===========================================================================
// Endian related

UINT32 BYTESWAP32(UINT32 u4Value);

// LINT_SUPPRESS_NEXT_EXPRESSION(129)
// INLINE UINT16 BYTESWAP16(UINT16 u2Value);

// LINT_SUPPRESS_NEXT_EXPRESSION(129)
// INLINE UINT16 BYTESWAP16(UINT16 u2Value)
// {
// 	return (UINT16)((u2Value >> 8) | (u2Value << 8));
// }

// HTONL: Convert an UINT32 from host byte order to network byte order
// NTOHL: Convert an UINT32 from network byte order to host byte order
// HTONS: Convert an UINT16 from host byte order to network byte order
// NTOHS: Convert an UINT16 from network byte order to host byte order

#ifdef __BIG_ENDIAN

#define BIG_ENDIAN
#define HTONL(x)	(x)
#define NTOHL(x)	(x)
#define HTONS(x)	(x)
#define NTOHS(x)	(x)

#else	// __BIG_ENDIAN

#define LITTLE_ENDIAN
#define HTONL(x)	BYTESWAP32((x))
#define NTOHL(x)	BYTESWAP32((x))
#define HTONS(x)	((UINT16)((((UINT16)(x)) >> 8) | (((UINT16)(x)) << 8)))
#define NTOHS(x)	((UINT16)((((UINT16)(x)) >> 8) | (((UINT16)(x)) << 8)))

#endif	// __BIG_ENDIAN


#endif  // X_HAL_926_H
