/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * Description:
 *      A static list, a list that maintains the index of an array to emulate
 *      the list behavior
 *
 *---------------------------------------------------------------------------*/

#ifndef X_LIST_H
#define X_LIST_H

#include "x_typedef.h"

//=====================================================================
// Type definitions

#define NULL_INDEX                      (-1)

typedef struct
{
    INT32           i4Size;
    INT32           i4Count;
    INT32           i4Head;
    INT32           i4Tail;
    INT32*          arNext;
    INT32*          arPrev;
} SLIST_T;


//=====================================================================
// Interface

extern BOOL SLIST_Init(SLIST_T* plist, INT32 u4ListSize);

extern BOOL SLIST_Release(SLIST_T* plist);

extern BOOL SLIST_IsEmpty(SLIST_T* plist);

extern BOOL SLIST_IsFull(SLIST_T* plist);

extern INT32 SLIST_GetCount(SLIST_T* plist);

extern INT32 SLIST_GetSize(SLIST_T* plist);

extern BOOL SLIST_IsValidIndex(SLIST_T* plist, INT32 i4Index);

extern INT32 SLIST_AddTail(SLIST_T* plist);

extern INT32 SLIST_AddHead(SLIST_T* plist);

extern INT32 SLIST_RemoveHead(SLIST_T* plist);

extern INT32 SLIST_RemoveTail(SLIST_T* plist);

extern BOOL SLIST_RemoveAt(SLIST_T* plist, INT32 i4Index);

extern BOOL SLIST_RemoveAll(SLIST_T* plist);

extern INT32 SLIST_GetHeadIndex(SLIST_T* plist);

extern INT32 SLIST_GetTailIndex(SLIST_T* plist);

extern INT32 SLIST_GetNextIndex(SLIST_T* plist, INT32 i4Index);

extern INT32 SLIST_GetPrevIndex(SLIST_T* plist, INT32 i4Index);



#endif  // X_LIST_H
