
PWD = $(shell pwd)

export BUILD_TARGET_OBJ_ROOT    = ${PWD}/OSS_Output/directfb
export VM_LINUX_ROOT            = ${PWD}/../../..
export CHILING_ROOT             = ${PWD}/../..
export OSS_OUTPUT               = ${BUILD_TARGET_OBJ_ROOT}

export ARCH             = arm
export ENABLE_CA9_NEON  = true

export TOOL_CHAIN            = 4.8.2
export USER_CROSS_COMPILE   ?= /mtkoss/gnuarm/neon_4.8.2_2.6.35_cortex-a9-ubuntu/x86_64/bin/armv7a-mediatek482_001_neon-linux-gnueabi-
export KERNEL_CROSS_COMPILE ?= /mtkoss/gnuarm/gcc-aarch64-linux-gnu-4.9.3-ubuntu/x86_64/bin/aarch64-linux-gnu-
export CLANG_PATH           ?= $(VM_LINUX_ROOT)/../android/q-sony/prebuilts/clang/host/linux-x86/clang-r353983b/bin/clang

export KERNEL_VER       ?= linux-4.19
export DIRECTFB_VERSION = 1.5.3

export ZLIB_VERSION     = 1.2.11
export PNG_VERSION      = 1.2.43
export FREETYPE_VERSION = 2.10.4
export JPEG_VERSION     = 6b

export TOP    = ${PWD}
export SOURCE = $(TOP)/directfb_$(DIRECTFB_VERSION)/Source
export KERNEL = $(TOP)/directfb_$(DIRECTFB_VERSION)/Kernel

export LINUX_ETC_PATH   = /mnt/vendor/linux_rootfs/etc
export LINUX_TMP_PATH   = /data/vendor/tmp
export DEFINES          = -DLINUX_ETC_PATH=\"$(LINUX_ETC_PATH)\" \
                          -DLINUX_TMP_PATH=\"$(LINUX_TMP_PATH)\"

export INCLUDES =   -I$(TOP)/MediaTek/mt53xx                        \
                    -I$(TOP)/MediaTek/gfx                           \
                    -I$(VM_LINUX_ROOT)/middleware/open/res_mngr/drv \
                    -I$(VM_LINUX_ROOT)/middleware/open/inc          \
                    -I$(VM_LINUX_ROOT)/middleware/open/os/inc       \
                    -I$(VM_LINUX_ROOT)/middleware/open/handle       \
                    -I$(VM_LINUX_ROOT)/middleware/open/

export CFLAGS   = -w
export CPPFLAGS = -w

export TARGET_IC        = mt5895
export KERNEL_CONFIG    = $(TARGET_IC)_android_smp_mod_defconfig
export KERNEL_OUT       = ${BUILD_TARGET_OBJ_ROOT}/../${KERNEL_VER}/${KERNEL_CONFIG}

export DFB_OSS_BUILD    = ${MAKECMDGOALS}

USER_BUILD_OPTIONS      = CC=${USER_CROSS_COMPILE}gcc CXX=${USER_CROSS_COMPILE}g++ AR=${USER_CROSS_COMPILE}ar STRIP=${USER_CROSS_COMPILE}strip
ifeq (${KERNEL_VER}, linux-4.9)
KERNEL_BUILD_OPTIONS    = CC=${KERNEL_CROSS_COMPILE}gcc CROSS_COMPILE=${KERNEL_CROSS_COMPILE} ARCH=arm64 KCFLAGS="-DCC_SUPPORT_ANDROID_L_RTC_WAKEUP -DCC_AOS_PMIC" KBUILD_MODPOST_WARN=y
else
KERNEL_BUILD_OPTIONS    = CC=${CLANG_PATH} CROSS_COMPILE=${KERNEL_CROSS_COMPILE} ARCH=arm64 KCFLAGS="-DCC_SUPPORT_ANDROID_L_RTC_WAKEUP -DCC_AOS_PMIC" KBUILD_MODPOST_WARN=y
endif

KERNEL_CONFIG_FILE      ?= kernel_config_uroboros_q

all:
ifneq ($(DFB_OSS_BUILD),modules)
	@/bin/echo -e "Build the Linux kernel common ..."
	rm -rf ${KERNEL_OUT}_modules
	mkdir -p $(KERNEL_OUT)_modules
	make -C ${CHILING_ROOT}/kernel/${KERNEL_VER} ${KERNEL_BUILD_OPTIONS} O=$(KERNEL_OUT)_modules bravia_defconfig
	make -C ${CHILING_ROOT}/kernel/${KERNEL_VER} ${KERNEL_BUILD_OPTIONS} O=$(KERNEL_OUT)_modules modules
else
	rm -rf ${BUILD_TARGET_OBJ_ROOT}
	@test -d $(VM_LINUX_ROOT)/target/open/mt53xx_com/53xx_com_driver || ( \
		ln -s $(VM_LINUX_ROOT)/target/open/mt53xx_com/driver $(VM_LINUX_ROOT)/target/open/mt53xx_com/53xx_com_driver \
	)
	@/bin/echo -e "Build the oss library: libdirectfb.so"
	mkdir -pv $(OSS_OUTPUT)/basic/lib
	make -C $(VM_LINUX_ROOT)/oss/library/gnuarm-4.8.2_neon_ca9/freetype/$(FREETYPE_VERSION)/pre-install/lib all
	make -C $(VM_LINUX_ROOT)/oss/library/gnuarm-4.8.2_neon_ca9/zlib/$(ZLIB_VERSION)/pre-install/lib all
	make -C $(VM_LINUX_ROOT)/oss/library/gnuarm-4.8.2_neon_ca9/png/$(PNG_VERSION)/pre-install/lib all
	make -C $(VM_LINUX_ROOT)/oss/library/gnuarm-4.8.2_neon_ca9/jpeg/$(JPEG_VERSION)/pre-install/lib all
	@/bin/echo -e "Build the DirectFB OBJs:"
	make ${USER_BUILD_OPTIONS} gen_obj
	make ${USER_BUILD_OPTIONS} libdirectfb.so
	cp ${BUILD_TARGET_OBJ_ROOT}/libdirectfb.so ${BUILD_TARGET_OBJ_ROOT}/../

	@/bin/echo -e "Build the oss library: fusion.ko"
ifeq (${KERNEL_VER}, linux-4.9)
	make ${KERNEL_BUILD_OPTIONS} O=$(KERNEL_OUT)_modules fusion.ko
else
	make ${KERNEL_BUILD_OPTIONS} O=$(KERNEL_OUT) fusion.ko
endif
	cp ${BUILD_TARGET_OBJ_ROOT}/fusion.ko ${BUILD_TARGET_OBJ_ROOT}/../
endif
