/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_handle.h 
 *  $RCSfile: $
 *  $Revision: #2 $
 *  $Date: 2015/08/08 $
 *  $Author: xiaojun.zhu $
 *  
 *  @par Description:
 *         This header file contains handle specific definitions, which are
 *         exported.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_HANDLE Handle Module
 *
 *  @ingroup groupMW
 *  @brief Provides external function to alloc/free/get handles
 *
 *  %MW Application and middleware all will use this module to alloc and free and get handles.
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/


#ifndef _U_HANDLE_H_
#define _U_HANDLE_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_common.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Specify handle data types */
#if defined(__aarch64__)
typedef UPTR    HANDLE_T;    /**<  handle defination      */
#else
typedef UINT32  HANDLE_T;    /**<  handle defination      */
#endif
typedef UINT16  HANDLE_TYPE_T;    /**<  type of handle     */

#define NULL_HANDLE  ((HANDLE_T) 0)           /**< Value of empty handle */

#define INV_HANDLE_TYPE  ((HANDLE_TYPE_T) 0)  /**< Value of invalid handle type */

/* Handle API return values */
#define HR_OK                   ((INT32)   0) /**< Call handle function ok */
#define HR_INV_ARG              ((INT32)  -1) /**< Input argment is invalid */
#define HR_INV_HANDLE           ((INT32)  -2) /**< Input handle is invalid */
#define HR_OUT_OF_HANDLES       ((INT32)  -3) /**< Input handle is outof handle array */
#define HR_NOT_ENOUGH_MEM       ((INT32)  -4) /**< Memory is not enough */
#define HR_ALREADY_INIT         ((INT32)  -5) /**< Handle is already inited */
#define HR_NOT_INIT             ((INT32)  -6) /**< Handle is not inited */
#define HR_RECURSION_ERROR      ((INT32)  -7) /**< Handle is already in freeing */
#define HR_NOT_ALLOWED          ((INT32)  -8) /**< Handle operation is not allowed */
#define HR_ALREADY_LINKED       ((INT32)  -9) /**< Handle is already linked in list */
#define HR_NOT_LINKED           ((INT32) -10) /**< Handle is not linked in list */
#define HR_FREE_NOT_ALLOWED     ((INT32) -11) /**< Handle is not allowed to free */
#define HR_INV_AUX_HEAD         ((INT32) -12) /**< Handle auxiliary head is invalid */
#define HR_INV_HANDLE_TYPE      ((INT32) -13) /**< Handle type is invalid */
#define HR_CANNOT_REG_WITH_CLI  ((INT32) -14) /**< Register handle debug module failed */


#endif /* _U_HANDLE_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */                                              /* end of groupMW_HANDLE */
/*----------------------------------------------------------------------------*/

